import 'package:flutter/material.dart';
import 'package:neo_toolbox/managers/lifecycle_manager/stoppable_service.dart';

/// LifeCycleManager stops and starts long running services when app changes state.
/// Reference: https://www.filledstacks.com/post/flutter-application-life-cycle-management/
///
/// Use the get_it library (https://pub.dev/packages/get_it) or Provider to inject services
/// to the servicesToManage list.
///
class LifeCycleManager extends StatefulWidget {
  final Widget child;
  final List<StoppableService> servicesToManage;
  LifeCycleManager({
    Key key,
    this.servicesToManage,
    this.child,
  }) : super(key: key);

  _LifeCycleManagerState createState() => _LifeCycleManagerState();
}

class _LifeCycleManagerState extends State<LifeCycleManager>
    with WidgetsBindingObserver {
  List<StoppableService> servicesToManage;

  @override
  Widget build(BuildContext context) {
    return widget.child;
  }

  @override
  void initState() {
    servicesToManage = widget.servicesToManage;
    super.initState();
    WidgetsBinding.instance.addObserver(this);
  }

  @override
  void dispose() {
    super.dispose();
    WidgetsBinding.instance.removeObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    servicesToManage.forEach((service) {
      if (state == AppLifecycleState.resumed) {
        service.start();
      } else {
        service.stop();
      }
    });
  }
}
