import 'package:flutter/cupertino.dart';
import 'package:provider/provider.dart';

/// ViewNotifierConsumer will create both notifier and cosumer in the same place.
/// The drawback of this method is that the whole widget tree will be rebuilt
/// whenever the notifier call the notifyListeners() method.
///
/// Reference:
/// https://www.filledstacks.com/post/flutter-provider-v3-architecture/
///
class ViewNotifierConsumer<T extends ChangeNotifier> extends StatefulWidget {
  final Widget Function(BuildContext context, T notifier, Widget child) builder;
  final T notifier;
  final Widget child;

  /// this function is used to load data or call async methods before build widget
  /// which is normally not allowed with StatelessWidget
  final Function(T) onViewInit;

  const ViewNotifierConsumer({
    Key key,
    this.builder,
    this.notifier,
    this.onViewInit,
    this.child,
  }) : super(key: key);

  @override
  _ViewNotifierConsumerState<T> createState() =>
      _ViewNotifierConsumerState<T>();
}

class _ViewNotifierConsumerState<T extends ChangeNotifier>
    extends State<ViewNotifierConsumer<T>> {
  T notifier;

  @override
  void initState() {
    notifier = widget.notifier;
    if (widget.onViewInit != null) {
      widget.onViewInit(notifier);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<T>(
      builder: (_) => notifier,
      child: Consumer<T>(
        builder: widget.builder,
        child: widget.child,
      ),
    );
  }
}
