import 'package:flutter/widgets.dart';
import 'package:neo_toolbox/responsive/sizing_information.dart';
import 'package:neo_toolbox/responsive/ui_utils.dart';

/// ResponsiveBuilder is used to get the screen size and the parent widget size.
///
/// Reference: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter/
///
class ResponsiveBuilder extends StatelessWidget {
  final Widget Function(
    BuildContext context,
    SizingInformation sizingInformation,
  ) builder;
  const ResponsiveBuilder({Key key, this.builder}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, boxConstraints) {
      var mediaQuery = MediaQuery.of(context);
      var sizingInformation = SizingInformation(
        deviceScreenType: getDeviceType(mediaQuery),
        screenSize: mediaQuery.size,
        localWidgetSize:
            Size(boxConstraints.maxWidth, boxConstraints.maxHeight),
      );
      return builder(context, sizingInformation);
    });
  }
}
