import 'package:flutter/widgets.dart';
import 'package:neo_toolbox/responsive/device_screen_type.dart';
import 'package:neo_toolbox/responsive/responsive_builder.dart';

/// ScreenTypeLayout will automatically choose the mobile, tablet, desktop based on
/// the current width of the screen
///
/// phone < 600 < tablet < 950 < desktop
///
/// Reference: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter/
///
/// ScreenTypeLayout will be combined with OrientationLayout to have full responsive app
/// (wrap the OrientationLayout with the ScreenTypeLayout)
///
///  ScreenTypeLayout => phone => phone landcapse, phone portrait
///                   => tablet => tablet landcapse, tablet portrait
///                   => desktop => desktop landcapse, desktop portrait
///
class ScreenTypeLayout extends StatelessWidget {
  // Mobile will be returned by default
  final Widget mobile;
  final Widget tablet;
  final Widget desktop;

  const ScreenTypeLayout(
      {Key key, @required this.mobile, this.tablet, this.desktop})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ResponsiveBuilder(builder: (context, sizingInformation) {
      // If sizing indicates Tablet and we have a tablet widget then return
      if (sizingInformation.deviceScreenType == DeviceScreenType.Tablet) {
        if (tablet != null) {
          return tablet;
        }
      }

      // If sizing indicates desktop and we have a desktop widget then return
      if (sizingInformation.deviceScreenType == DeviceScreenType.Desktop) {
        if (desktop != null) {
          return desktop;
        }
      }

      // Return mobile layout if nothing else is supplied
      return mobile;
    });
  }
}
