import 'package:flutter/material.dart';

/// NavigationService is used to avoid the need of BuilderContext when
/// you want to decouple UI and business logic.
///
/// The NavigationService object should be available to the caller by using
/// provider or other dependency injestion libraries (i.e. GetIt https://pub.dev/packages/get_it).
///
/// Use this service by setting the 'navigatorKey' property of the MaterialApp
/// or the 'key' in the Navigator widget to the navigatorKey of NavigationService.
///
///
class NavigationService {
  final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

  bool pop([result]) {
    return navigatorKey.currentState.pop(result);
  }

  Future<dynamic> pushReplacementNamed(String routeName,
      {dynamic arguments, dynamic result}) {
    return navigatorKey.currentState.pushReplacementNamed(
      routeName,
      arguments: arguments,
      result: result,
    );
  }

  Future<dynamic> pushNamed(String routeName, {dynamic arguments}) {
    return navigatorKey.currentState.pushNamed(
      routeName,
      arguments: arguments,
    );
  }

  Future<dynamic> pushNamedAndRemoveUntil(String route, String baseRoute,
      {dynamic arguments}) {
    var predicate = baseRoute == ""
        ? (Route<dynamic> route) => false
        : ModalRoute.withName(baseRoute);

    return navigatorKey.currentState.pushNamedAndRemoveUntil(
      route,
      predicate,
      arguments: arguments,
    );
  }

  void popUntil(String route) {
    return navigatorKey.currentState.popUntil(ModalRoute.withName(route));
  }
}
